{ Copyright (C) 2012 H Page-Clark

  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.

  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
  MA 02111-1307, USA. }

unit browser_utils;

{$mode objfpc}{$H+}

interface

uses
  Classes, Forms, Menus, Buttons, StdCtrls, ExtCtrls, ActnList, MaskEdit, grids,
  CheckLst, PairSplitter, ColorBox, ValEdit, ComCtrls, Controls, PopupNotifier,
  Dialogs, ExtDlgs, printersDlgs, DbCtrls, DBGrids, LResources, LazHelpHTML,
  AsyncProcess, UTF8Process, process, simpleipc, XMLConf, eventlog,
  ServiceManager, Spin, Arrow, Calendar, EditBtn, FileCtrl, XMLPropStorage,
  IniPropStorage, BarChart, ButtonPanel, ShellCtrls, IDEWindowIntf, db,
  BufDataset, memds, SdfData, dbf,
  SynEdit, SynMemo, SynCompletion, SynMacroRecorder, SynExportHTML,
  SynPluginSyncroEdit, SynHighlighterBat, SynHighlighterCpp, SynHighlighterHTML,
  SynHighlighterDiff, SynHighlighterIni, SynHighlighterJava, SynHighlighterPas,
  SynHighlighterPerl, SynHighlighterXML, SynHighlighterLFM,
  synhighlighterunixshellscript, SynHighlighterCss, SynHighlighterPHP,
  SynHighlighterTeX, SynHighlighterSQL, SynHighlighterPython, SynHighlighterVB,
  SynHighlighterAny, SynHighlighterMulti, SynHighlighterPo,
  DividerBevel, ExtendedNotebook, ListFilterEdit, TreeFilterEdit,
  sqldb, oracleconnection, odbcconn, pqconnection, mysql40conn, mysql41conn,
  mysql50conn, mysql51conn, sqlite3conn, IBConnection,
  RTTICtrls, RTTIGrids,
  ipfilebroker, IpHtml,
  TAGraph, TASources, TADbSource, TATools, TATransformations, TAStyles,
  TALegendPanel, TANavigation, TAIntervalSources, TAChartListbox,
  TAChartExtentLink, TAChartImageList;

type
  TPalettePage = (ppStandard, ppAdditional, ppCommon, ppDialogs, ppDataControls,
                  ppSystem, ppMisc, ppDataAccess, ppSynEdit, ppLazControls,
                  ppRTTI, ppIPro, ppChart, ppSQLdb);
const
  MaxComponentsOnAPage = 28; // TSynEdit page
  PageNames: array[TPalettePage] of shortstring =
    ('Standard', 'Additional', 'Common Controls', 'Dialogs', 'Data Controls',
     'System', 'Misc', 'Data Access', 'SynEdit', 'LazControls',
     'RTTI', 'IPro', 'Chart', 'SQLdb');

function GetComponentClass(aPage: TPalettePage; anIndex: word): TComponentClass;
function GetAncestorCount(aClass: TClass): integer;

implementation

function GetComponentClass(aPage: TPalettePage; anIndex: word): TComponentClass;
begin
  case aPage of
   ppStandard: case anIndex of
                1:  result := TMainMenu;
                2:  result := TPopupMenu;
                3:  result := TButton;
                4:  result := TLabel;
                5:  result := TEdit;
                6:  result := TMemo;
                7:  result := TToggleBox;
                8:  result := TCheckBox;
                9:  result := TRadioButton;
                10: result := TListBox;
                11: result := TComboBox;
                12: result := TScrollBar;
                13: result := TGroupBox;
                14: result := TRadioGroup;
                15: result := TCheckGroup;
                16: result := TPanel;
                17: result := TFrame;
                18: result := TActionList;
                else result := nil;
               end;
   ppAdditional: case anIndex of
                1:  result := TBitBtn;
                2:  result := TSpeedButton;
                3:  result := TStaticText;
                4:  result := TImage;
                5:  result := TShape;
                6:  result := TBevel;
                7:  result := TPaintBox;
                8:  result := TNotebook;
                9:  result := TLabeledEdit;
                10: result := TSplitter;
                11: result := TTrayIcon;
                12: result := TMaskEdit;
                13: result := TCheckListBox;
                14: result := TScrollBox;
                15: result := TApplicationProperties;
                16: result := TStringGrid;
                17: result := TDrawGrid;
                18: result := TPairSplitter;
                19: result := TColorBox;
                20: result := TColorListBox;
                21: result := TValueListEditor;
                else result := nil;
               end;
   ppCommon:  case anIndex of
      		1:  result := TTrackBar;
                2:  result := TProgressBar;
                3:  result := TTreeView;
                4:  result := TListView;
                5:  result := TStatusBar;
                6:  result := TToolBar;
                7:  result := TUpDown;
                8:  result := TPageControl;
                9:  result := TTabControl;
                10: result := THeaderControl;
                11: result := TImageList;
                12: result := TPopupNotifier;
                else result := nil;
               end;
   ppDialogs: case anIndex of
      		1:  result := TOpenDialog;
                2:  result := TSaveDialog;
                3:  result := TSelectDirectoryDialog;
                4:  result := TColorDialog;
                5:  result := TFontDialog;
                6:  result := TFindDialog;
                7:  result := TReplaceDialog;
                8:  result := TOpenPictureDialog;
                9:  result := TSavePictureDialog;
                10: result := TCalendarDialog;
                11: result := TCalculatorDialog;
                12: result := TPrinterSetupDialog;
                13: result := TPrintDialog;
                14: result := TPageSetupDialog;
                else result := nil;
               end;
   ppDataControls: case anIndex of
                1:  result := TDBNavigator;
                2:  result := TDBText;
                3:  result := TDBEdit;
                4:  result := TDBMemo;
                5:  result := TDBImage;
                6:  result := TDBListBox;
                7:  result := TDBLookupListBox;
                8:  result := TDBComboBox;
                9:  result := TDBLookupComboBox;
                10: result := TDBCheckBox;
                11: result := TDBRadioGroup;
                12: result := TDBCalendar;
                13: result := TDBGroupBox;
                14: result := TDBGrid;
                else result := nil;
               end;
   ppSystem:  case anIndex of
      		1:  result := TTimer;
                2:  result := TIdleTimer;
                3:  result := TLazComponentQueue;
                4:  result := THTMLHelpDatabase;
                5:  result := THTMLBrowserHelpViewer;
                6:  result := TAsyncProcess;
                7:  result := TProcessUTF8;
                8:  result := TProcess;
                9:  result := TSimpleIPCClient;
                10: result := TSimpleIPCServer;
                11: result := TXMLConfig;
                12: result := TEventLog;
                13: result := TServiceManager;
                else result := nil;
               end;
   ppMisc: case anIndex of
   		1:  result := TColorButton;
                2:  result := TSpinEdit;
                3:  result := TFloatSpinEdit;
                4:  result := TArrow;
                5:  result := TCalendar;
                6:  result := TEditButton;
                7:  result := TFileNameEdit;
                8:  result := TDirectoryEdit;
                9:  result := TDateEdit;
                10: result := TCalcEdit;
                11: result := TFileListBox;
                12: result := TFilterComboBox;
                13: result := TXMLPropStorage;
                14: result := TIniPropStorage;
                15: result := TBarChart; // deprecated
                16: result := TButtonPanel;
                17: result := TShellTreeView;
                18: result := TShellListView;
                19: result := TIDEDialogLayoutStorage;
                else result := nil;
               end;
   ppDataAccess: case anIndex of
                1:  result := TDataSource;
                2:  result := TBufDataset;
                3:  result := TMemDataset;
                4:  result := TSdfDataSet;
                5:  result := TFixedFormatDataSet;
                6:  result := TDbf; // deprecated
                // 7:  result := TSqlite3Dataset;
                else result := nil;
               end;
   ppSynEdit: case anIndex of
                1:  result := TSynEdit;
                2:  result := TSynMemo;
                3:  result := TSynCompletion;
                4:  result := TSynAutoComplete;
                5:  result := TSynMacroRecorder;
                6:  result := TSynExporterHTML;
                7:  result := TSynPluginSyncroEdit;
                8:  result := TSynPasSyn;
                9:  result := TSynFreePascalSyn;
                10: result := TSynCppSyn;
                11: result := TSynJavaSyn;
                12: result := TSynPerlSyn;
                13: result := TSynHTMLSyn;
                14: result := TSynXMLSyn;
                15: result := TSynLFMSyn;
                16: result := TSynDiffSyn;
                17: result := TSynUNIXShellScriptSyn;
                18: result := TSynCssSyn;
                19: result := TSynPHPSyn;
                20: result := TSynTeXSyn;
                21: result := TSynSQLSyn;
                22: result := TSynPythonSyn;
                23: result := TSynVBSyn;
  		24: result := TSynAnySyn;
                25: result := TSynMultiSyn;
                26: result := TSynBatSyn;
                27: result := TSynPoSyn;
                MaxComponentsOnAPage: result := TSynIniSyn;
                else result := nil;
               end;
   ppLazControls:  case anIndex of
      		1:  result := TDividerBevel;
                2:  result := TExtendedNotebook;
                3:  result := TListFilterEdit;
                4:  result := TTreeFilterEdit;
                //5:  result := TCodyTreeView;
                else result := nil;
               end;
   ppSQLdb:  case anIndex of
      		1:  result := TSQLQuery;
                2:  result := TSQLTransaction;
                3:  result := TSQLScript;
                4:  result := TSQLConnector;
                5:  result := TSQLConnection;
                6:  result := TOracleConnection;
                7:  result := TODBCConnection;
                8:  result := TMySQL40Connection;
                9:  result := TMySQL41Connection;
                10: result := TMySQL50Connection;
                11: result := TMySQL51Connection;
                12: result := TSQLite3Connection;
                13: result := TIBConnection;
                else result := nil;
             end;
   ppRTTI: case anIndex of
                1:  result := TTIEdit;
                2:  result := TTIComboBox;
                3:  result := TTIButton;
                4:  result := TTICheckBox;
                5:  result := TTILabel;
                6:  result := TTIGroupBox;
                7:  result := TTIRadioGroup;
                8:  result := TTICheckGroup;
                9:  result := TTICheckListBox;
                10: result := TTIListBox;
                11: result := TTIMemo;
                12: result := TTICalendar;
                13: result := TTIImage;
                14: result := TTIFloatSpinEdit;
                15: result := TTISpinEdit;
                16: result := TTITrackBar;
                17: result := TTIProgressBar;
                18: result := TTIMaskEdit;
                19: result := TTIColorButton;
                20: result := TMultiPropertyLink;
                21: result := TTIPropertyGrid;
                22: result := TTIGrid;
                else result := nil;
            end;
   ppIPro: case anIndex of
    	        1: result := TIpFileDataProvider;
                2: result := TIpHtmlPanel;
                else result := nil ;
           end;
   ppChart: case anIndex of
                1:  result := TChart;
                2:  result := TListChartSource;
                3:  result := TRandomChartSource;
                4:  result := TUserDefinedChartSource;
                5:  result := TCalculatedChartSource;
                6:  result := TDbChartSource;
                7:  result := TChartToolset;
                8:  result := TChartAxisTransformations;
                9:  result := TChartStyles;
                10: result := TChartLegendPanel;
                11: result := TChartNavScrollBar;
                12: result := TChartNavPanel;
                13: result := TIntervalChartSource;
                14: result := TDateTimeIntervalChartSource;
                15: result := TChartListBox;
                16: result := TChartExtentLink;
                17: result := TChartImageList;
                else result := nil;
             end;

   else result := nil;
  end;
end;

function GetAncestorCount(aClass: TClass): integer;
begin
  result := 0;
  if not Assigned(aClass.ClassParent)
    then Exit
  else
    begin
      while Assigned(aClass.ClassParent) do
        begin
          inc(result);
          aClass:= aClass.ClassParent;
        end;
    end;
end;

end.

