{ Copyright (C) 2012 H Page-Clark

  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.

  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
  MA 02111-1307, USA. }

unit anagram_main;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Dialogs, StdCtrls, ExtCtrls;

type

  { TForm1 }

  TForm1 = class(TForm)
    BFasterAnagrams: TButton;
    BSave: TButton;
    BSlowAnagrams: TButton;
    EWord: TLabeledEdit;
    LAnagramCount: TLabel;
    LBWords: TListBox;
    LElapsedTime: TLabel;
    LProgress: TLabel;
    procedure BFasterAnagramsClick(Sender: TObject);
    procedure BSaveClick(Sender: TObject);
    procedure BSlowAnagramsClick(Sender: TObject);
    procedure EWordChange(Sender: TObject);
  private
    wordCount: int64;
    start: TDateTime;
    ss: TStringStream;
    procedure Exchange(var a, b: Char);
    function Factorial(anInt: integer): int64;
    procedure GenerateAnagramsFaster(aWord: string; charPos: integer);
    procedure GenerateAnagramsPoorly(aWord: string; charPos: integer);
  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TForm1 }

procedure TForm1.EWordChange(Sender: TObject);
var w: integer;
    f: int64;

    function Plural(x: int64): char;
    begin
      Result:= #0;
      if (x > 1) then Result := 's';
    end;
begin
  w:= EWord.GetTextLen;
  f:= Factorial(w);
  LAnagramCount.Caption:= Format('"%s" will generate %s anagram%s',
                                 [EWord.Text, FormatFloat(',#',f), Plural(f)]);
  LProgress.Caption:= EmptyStr;
  LElapsedTime.Caption:= EmptyStr;
end;

procedure TForm1.Exchange(var a, b: Char);
var tmp: Char;
begin
   tmp := a;
   a := b;
   b := tmp;
end;

procedure TForm1.BSlowAnagramsClick(Sender: TObject);
begin
  LBWords.Items.Clear;
  LBWords.Items.Add('[Slower]');
  wordCount:= 0;
  start:= Now;
  LBWords.Items.BeginUpdate;
  GenerateAnagramsPoorly(EWord.Text, 1);
  LBWords.Items.EndUpdate;
  LElapsedTime.Caption:= 'Elapsed time: '+FormatDateTime('n"m" s:z"s"',Now-start);
end;

procedure TForm1.BFasterAnagramsClick(Sender: TObject);
begin
  LBWords.Items.Clear;
  wordCount:= 0;
  ss := TStringStream.Create('[Faster]'+LineEnding);
  ss.Seek(0, soFromEnd);
  start:= Now;
  LBWords.Items.BeginUpdate;
  try
    GenerateAnagramsFaster(EWord.Text, 1);
    LProgress.Caption:= 'Loading listbox from stream...';
    Application.ProcessMessages;
    ss.Seek(0, soFromBeginning);
    LBWords.Items.LoadFromStream(ss);
  finally
    ss.Free;
  end;
  LBWords.Items.EndUpdate;
  LElapsedTime.Caption:='Elapsed time: '+FormatDateTime('n"m" s:z"s"',Now-start);
end;

procedure TForm1.BSaveClick(Sender: TObject);
begin
  LBWords.Items.SaveToFile(EWord.text+'.txt');
end;

function TForm1.Factorial(anInt: integer): int64;
begin
  result := -1;
  if (anInt < 0) then Exit;
  case anInt of
   0, 1: Result:= 1;
   else try
         Result:= anInt*Factorial(Pred(anInt));
        except
         on E:EIntOverflow do
          ShowMessageFmt(E.Message+'%sin Factorial function, parameter = %d',
                         [LineEnding, anInt]);
        end;
  end;
end;

procedure TForm1.GenerateAnagramsFaster(aWord: string; charPos: integer);
var le, p: integer;
begin
  le := Length(aWord);
  if (charPos >= le) then
    begin
    ss.WriteString(aWord + LineEnding);
    inc(wordCount);
    if (wordCount mod 10) = 0 then
      LProgress.Caption:= Format('%d anagrams generated',[wordCount]);
    Application.ProcessMessages;
    end
  else for p:= charPos to le do
   begin
    Exchange(aWord[p], aWord[charPos]);
    GenerateAnagramsFaster(aWord, Succ(charPos));
    Exchange(aWord[p], aWord[charPos]);
   end;
end;

procedure TForm1.GenerateAnagramsPoorly(aWord: string; charPos: integer);
var le, p: integer;
begin
  le := Length(aWord);
  if (charPos >= le) then
    begin
    LBWords.Items.Text := Format('%s%s',[LBWords.Items.Text, aWord]);
    inc(wordCount);
    if (wordCount mod 10) = 0 then
      LProgress.Caption:= Format('%d anagrams generated',[wordCount]);
    Application.ProcessMessages;
    end
  else for p:= charPos to le do
   begin
    Exchange(aWord[p], aWord[charPos]);
    GenerateAnagramsPoorly(aWord, Succ(charPos));
    Exchange(aWord[p], aWord[charPos]);
   end;
end;

end.

